#!/usr/bin/env python3
import os, sys, logging
from docx import Document
from docx.shared import Inches
import imghdr, struct, math
from .LogGen import set_up_logging
from .XtraFunctions import GetDate, NumberMe, cli_progress_test

logger = logging.getLogger(__name__)

class pics2word:

    def __init__(self):
        # set default values on instantiation, until changed with CL args
        logger.debug("Running constructor functions")
        self.SetPath()
        self.SetTitle() 
        self.SetPicWidth()
        self.SetPicHeight()
        self.GetPicsInPath()
        self.SetFormat()
        self.SetTableWidth()

    def SetPath(self, Path=os.getcwd()):
        # Default path is the current working directory on the command line
        logger.info("Getting pictures from %s." % Path)
        self.path = os.path.abspath(Path)

    def SetTitle(self, title="PhotoDoc", date='y'):
        # if date begions with 'y', append title with date
        if date[0] == 'y':
            Today = GetDate()
            self.title = title + "_" + str(Today)
        else:
            self.title = title

    def SetPicWidth(self,Width=4):
        # TODO set a default!
        self.width = Width

    def SetPicHeight(self,Height=4):
        # TODO set a default!
        self.height = Height

    def SetTableWidth(self, Columns=2):
        # TODO set a default!
        self.tablecolumns = Columns
    
    def SetFormat(self, format="normal"):
        logger.debug("Setting format to %s." % format)
        if format[0].lower() == 't':
            self.format = "table"
        elif format[0].lower() == 'n':
            self.format = "normal"
        else:
            raise ValueError("Please enter a valid format for '-f' i.e. \"Normal\" or \"Table\"")

    def WriteDoc(self):
        if self.format[0].lower() == 't':
            logger.info("Writing to table.")
            self.WriteTable()
        else:
            logger.info("Writing to document normally.")
            self.WriteNormal()

    def WriteNormal(self):
        logger.info("Creating word document.")
        document = Document()
        p = document.add_paragraph()
        Path = self.path
        # Todo check if numbered and sort appropriately
        logger.debug("Sorting list of %s pictures." % len(self.pics))
        PicList = NumberMe(self.pics) # Sort pics into an order
        i=0
        for Pic in PicList:
            logger.debug("Writing picture: %s." % Pic)
            FullImageandPath = os.path.join(Path,Pic)
            r = p.add_run()
            logger.debug("Checking if %s is portrait." % Pic)
            isPortrait = self.IsPortrait(FullImageandPath)
            logger.debug("Adding %s to file." % Pic)
            if isPortrait:
                r.add_picture(FullImageandPath,height=Inches(self.height))
            else:
                r.add_picture(FullImageandPath,width=Inches(self.width))
            logger.debug("Adding description: %s." % Pic.split('.')[0])
            p.add_run("\n"+Pic.split('.')[0]+"\n")
            # update progress
            logger.debug("writing loading bar picture %s of %s." % (i, len(PicList)))
            cli_progress_test(cur_val=i,end_val=len(PicList),suffix=("writing loading bar picture %s of %s." % (i, len(PicList))))
            i += 1
        logger.info("Saving document as %s.docx" % self.title)
        document.save(self.title + '.docx')

    def WriteTable(self):
        logger.info("Creating word document.")
        document = Document()
        Path = self.path
        logger.debug("Sorting list of %s pictures." % len(self.pics))
        PicList = NumberMe(self.pics) # Sort pics into an order
        logger.info("Calculating number of rows.")
        numRows = self.GetNumberofRows()
        logger.info("Adding table of %s rows and %s columns." % (numRows, self.tablecolumns))
        table = document.add_table(rows=numRows, cols=self.tablecolumns)
        i=0
        Row_Index = 0 # Resets every iteration
        # list[start:stop:step] pastes the picture in every 2nd cell
        for row in table.rows[::2]:
            Col_Index = 0 # Resets every iteration
            for cell in row.cells:
                for paragraph in cell.paragraphs:
                    try:
                        Pic = PicList[i]
                        logger.debug("Writing %s in row %s cell %s paragraph %s." % (Pic, row, cell, paragraph))
                        FullImageandPath = os.path.join(str(Path),str(Pic))
                        r = paragraph.add_run()
                        logger.debug("Checking if %s is portrait." % Pic)
                        isPortrait = self.IsPortrait(FullImageandPath)
                        logger.debug("Adding %s to file." % Pic)
                        if isPortrait:
                            r.add_picture(FullImageandPath,height=Inches(self.height))
                        else:
                            r.add_picture(FullImageandPath,width=Inches(self.width))
                        # Offset 1 row down and add description
                        logger.debug("Writing description %s in row %s, column %s" % (Pic.split('.')[0], row, cell))
                        table.cell(row_idx=Row_Index + 1,col_idx=Col_Index).text = Pic.split('.')[0]
                        # Update user of progress
                        logger.debug("writing loading bar picture %s of %s." % (i, len(PicList)))
                        cli_progress_test(cur_val=i,end_val=len(PicList),suffix=("writing loading bar picture %s of %s." % (i, len(PicList))))
                    except IndexError:
                        # we incur an index error at the end of the picture list
                        # hence, we will simply pass and do nothing with the remaining empty cells
                        logger.warning("Index Error on picture %s indicating that there are remaining cells but no new pictures." % Pic)
                        pass
                    except:
                        print("Error in copying picture to table: %s" % sys.exc_info()[0])
                        logger.error("Error in copying picture to table: %s" % sys.exc_info()[0])
                        raise
                Col_Index += 1
                i += 1
            Row_Index += 2
        logger.info("Saving document as %s.docx" % self.title)
        document.save(self.title + '.docx')
    
    def GetPicsInPath(self):
        self.pics = []
        ValidExtList = [".jpg",".jpeg",".png",".bmp",".gif",".JPG",".JPEG",".PNG",".BMP",".GIF"]
        for file in os.listdir(self.path):
            for ValidExt in ValidExtList:
                if file.endswith(ValidExt):
                    logger.debug("Adding %s to PicList" % file)
                    self.pics.append(file)
        return self.pics

    def IsPortrait(self, fname):
        """Determine the image type of fhandle and return its size."""
        with open(fname, 'rb') as fhandle:
            head = fhandle.read(24)
            if len(head) != 24:
                return
            if imghdr.what(fname) == 'png':
                check = struct.unpack('>i', head[4:8])[0]
                if check != 0x0d0a1a0a:
                    return
                width, height = struct.unpack('>ii', head[16:24])
            elif imghdr.what(fname) == 'gif':
                width, height = struct.unpack('<HH', head[6:10])
            elif imghdr.what(fname) == 'jpeg':
                try:
                    fhandle.seek(0) # Read 0xff next
                    size = 2
                    ftype = 0
                    while not 0xc0 <= ftype <= 0xcf:
                        fhandle.seek(size, 1)
                        byte = fhandle.read(1)
                        while ord(byte) == 0xff:
                            byte = fhandle.read(1)
                        ftype = ord(byte)
                        size = struct.unpack('>H', fhandle.read(2))[0] - 2
                    # We are at a SOFn block
                    fhandle.seek(1, 1)  # Skip `precision' byte.
                    height, width = struct.unpack('>HH', fhandle.read(4))
                except Exception: #IGNORE:W0703
                    return
            else:
                return
            if width/height > 1:
                return False
            else:
                return True  

    def GetNumberofRows(self):
        cols = self.tablecolumns
        NumofPics = len(self.pics)
        return int(math.ceil(NumofPics / cols)) * 2



from operator import itemgetter
import datetime, sys, logging
from .LogGen import set_up_logging

logger = logging.getLogger(__name__)

def GetDate():
        logger.debug("Setting the date.")
        return datetime.date.today().strftime("%d%b%Y") # i.e. 15Feb2018

def NumberMe(List):
    #TODO This is creating silly errors such as:
    #FileNotFoundError: [Errno 2] No such file or directory: 
    #"/home/tom/Pictures/Wallpapers/['FT81c6h']"
    # As this is saving a list in a list, even though the list only has one item.
    # Need to 

        WordNum = []
        for item in List:
            SubList = ['','','']
            String = item.split('.')[0]# get value to the left of ".jpg" (or whichever)
            ext = item.split('.')[1]
            Num = String[len(String.rstrip('0123456789')):]
            # Add to list
            SubList[0] = str(String)
            try:
                SubList[1] = int(Num)
            except ValueError:
                pass # not all values contain numbers
            SubList[2] = str(ext)
            # Add SubList to Main List
            WordNum.append(SubList)

        # Sort by number, which has the index of 1 on each sublist
        WordNum.sort(key=itemgetter(1)) 

        NumberedList = []
        for word in WordNum:
            # Delete the number index so we have a clean list
            del word[1]
            NumberedList.append('.'.join(word)) #word = '.'.join(word) 
        
        for item in NumberedList:
            print(item)

        return NumberedList

def cli_progress_test(cur_val, end_val, bar_length=60, suffix=''):
    
    filled_len = int(round(bar_length * cur_val / float(end_val)))

    percents = round(100.0 * cur_val / float(end_val), 1)
    bar = '=' * filled_len + '-' * (bar_length - filled_len)

    print('[%s] %s%s ...%s\r\n' % (bar, percents, '%', suffix), flush=True)


#!/usr/bin/env python3
import logging, datetime

logger = logging.getLogger(__name__)

def logArg(x):
    return {
        'talk': logging.DEBUG,
        'info': logging.INFO,
        'quiet': logging.NOTSET
    }.get(x, logging.NOTSET) 
    # default = NOTSET

def set_up_logging(arg):
    level = logArg(arg)

    date = datetime.date.today().strftime("%d%b%Y")
    LogName = "Pics2WordLog_" + date + ".txt"
    logging.basicConfig(filename=LogName,level=level,format='%(asctime)s %(levelname)s: %(message)s', datefmt='%d/%m/%Y %I:%M:%S %p')

    ch = logging.StreamHandler()
    logger.addHandler(ch)


#!/usr/bin/env python3
import os, sys, logging, pickle
from .pics2word import *
from .LogGen import set_up_logging
from .help import help

logger = logging.getLogger(__name__)

def remDictKey(d, key):
    '''Returns a new dictionary with a key-value pair removed'''
    logger.debug("removing %s from arg list dictionary" % key)
    new_d = d.copy()
    new_d.pop(key)
    return new_d

# Method to parse command line arguements into command-value pairs
def getopts(argv):
    opts = {}  # Empty dictionary to store key-value pairs.
    while argv:  # While there are arguments left to parse...
        if argv[0][0] == '-':  # Found a "-name value" pair.
            logging.debug("Adding %s as a value to %s command." % (argv[1], argv[0]))
            opts[argv[0]] = argv[1]  # Add key and value to the dictionary.
        argv = argv[1:]  # Reduce the argument list by copying it starting from index 1.
    return opts

def main():
    #Arglist passed as immutable for key to dict 
    set_up_logging(tuple(sys.argv))
    IniArgs = getopts(sys.argv)
    # Load / save arg template
    if '-m' in IniArgs: 
        mod = IniArgs['-m']
        try:
            logger.info("Found module %s. Loading arg list." % mod)
            myargs = pickle.load(open(mod+".p", "rb"))
        except:
            # Assumes the file does not exist so sets one up
            logger.info("Module %s not found. creating module called %s" % (mod,mod))
            myargs = remDictKey(IniArgs, '-m') # dont resave save function
            pickle.dump(myargs, open(mod+".p", "wb"))
    else:
        myargs = IniArgs

    Doc = pics2word()  
    if '-h' in myargs:
        help(myargs['-h'])
    if '-P' in myargs:
        # Override the default path
        Doc.SetPath(myargs['-P'])
    if '-f' in myargs:
        # Set as table or default
        Doc.SetFormat(format=myargs['-f'])
    if '-T' in myargs:
        # Set a title to override the default
        Doc.SetTitle(title=myargs['-T'],date=myargs['-Td'])
    if '-pw' in myargs:
        # Override the default picture width
        Doc.SetPicWidth(float(myargs['-pw']))
    if '-ph' in myargs:
        # Override the default picture height
        Doc.SetPicHeight(float(myargs['-ph'])) 
    if '-tw' in myargs:
        if Doc.format[0] != 't' :
            raise ValueError("Must enable table format to format table width!")
        else:
            Doc.SetTableWidth(int(myargs['-tw']))
    
    # after all optional parameters have been changed and not asked for help, then write document.
    if '-h' not in myargs:
        Doc.WriteDoc()

if __name__ == '__main__':
    main()



def help(arg):
    Message = {}
    Message['help'] = '''Usage: pics2word [-command] [value]
Options:
\t-h\t- Pass "help" to print this help message to the terminal.
\t\t  Pass the name of a command below without the '-' for more informatio about that command.
\t\t  Pass "?" if you cant find what you are looking for or having issues.
\t-P\t- Pass an alternative path to be used. i.e. \"C:\\\\Pictures\\\". Defaults to current directory.
\t-f\t- format pictures. pass either "normal" or "table". Defaults to normal. 
\t-T\t- Override the default title. Defaults to PhotoDoc_<current date> (See Td, below).
\t-Td\t- Choose to append the title with the current date. Options: \"y\" or \"n\". Defaults to \"y\".
\t-pw\t- Set the width of imported pictures in inches. Defaults to 4 inches
\t-ph\t- Set the height of imported pictures in inches. Defaults to 4 inches
\t-tw\t- Set the number of columns used in table format. Note: table format must be enabled! Defaults to 2.
\t-m\t- Module. Allows the creation of templates. 

Commands may be passed as command-value pairs in any order.
All commands are optional and the defaults will be used if no commands are given.

Example: pics2word -P \"C:\\\\Pictures\\\" -T Report -Td n -f table\n'''

    Message['p'] = '''Name: Path command
Function: This allows the user to use pictures from an alternative directory.
Options: Any valid path. i.e. \"C:\\\\user\\Joe Bloggs\\pictures\"
Default: In the absense of any path being passed, the program will 
\t  use the current directory of the terminal running it.'''

    Message['f'] = '''Name: Format command
Function: Allows the user to define the format that the pictures are presented.
\t  Normal simply copies the picture into the table with the title on the next line
\t  Table will copy the pictures into a table with the title in the cell beneath.
Options: "normal" and "table" are valid options.
Default: "normal"'''

    Message['t'] = '''Name: Title command
Function: Allows the user to specify a title for the saved word document.
\t  Note: If this command is used then a value for 'Td' MUST be given. 
Options: If giving a name with spaces then you will need to pass the name in quotes, thus: "my report"
Default: "PhotoDoc_<date>" where <date> is the current date.'''

    Message['td'] = '''Name: Title Date command
Function: Gives the user the option to append the current date at the end of the title of the word document.
\t  Note: If a custom title is used then a value for 'Td' MUST be given.
\t  Example date format is produced thus: "09Mar2018"  For 9th March 2018
Options: 'y' or 'n' for yes or no.
Default: 'y' as part of the default title. Otherwise the user will have to give an option.'''

    Message['pw'] = '''Name: Picture Width
Function: Allows the user to define the width of the picture in inches.
\t  Important: Aspect ratio is preserved. This option restricts the width of landscape pictures only.
\t  for further control it is recommended that the an option for 'ph' also be passed. 
Options: Any positive number. Recommended values of 5 for normal usage, 3 for table usage (2 columns)
Default: 4 inches'''

    Message['ph'] = '''Name: Picture Height 
Function: Allows the user to define the height of the picture in inches.
\t  Important: Aspect ratio is preserved. This option restricts the height of portrait pictures only.
\t  for further control it is recommended that the an option for 'pw' also be passed. 
Options: Any positive number. Recommended values of 5 for normal usage, 2.5 for table usage (2 columns)
Default: 4 inches'''

    Message['tw'] = '''Name: Table Width
Function: Define the number of columns in the table.
\t  Important: The "table" format MUST be specified for this command to take effect.
Options: Any positive integer. Recommended values between 1 & 3 for portrait A4.
Default: 2 columns'''

    Message['m'] = '''Name: Module command
Function: This command allows the user to define and use templates Pass -m "report" to load the arguments saved in that module. 
\t\t  If the module name does not exist then one will be created under the passed name using the passed arguments as saved values.
\t\t  Note that when loading the template, arguments passed with a loaded module will overwrite the template values but do not save permanently.
Options: A valid name for a module must be used. If you are looking to load a module, make sure it exists in the current directory.
Default: If no modifications to the commands are given, then the default values set by the program will be used.'''

    Message['?'] = '''Please report all issues to: https://github.com/Ghostom998/pics2word/issues
Raise a new issue then:
- Please describe the issue as accurately as possible with the ouput from the log file and any error message given in the terminal window.
- Please describe exactly what you did to trigger the event including the commands that were passed
- Note the operating system, version of python used and pics2word version
- Include any further relavent information you deem useful.'''

    try:
        print(Message[arg.lower()])
    except:
        print("Please enter a valid option.\nRemember to pass the right command without the hyphen.")


#!/usr/bin/env python3
#setup.py
import sys, os
from distutils.core import setup
from setuptools import setup

__version__ = "0.9.3"

def readme():
    with open('README.md') as f:
        return f.read()

setup(
    name = "pics2word",
    description='A command line program to copy pictures into a word document.',
    version=__version__,
    url='https://github.com/Ghostom998/pics2word.git',
    author='Thomas Roberts',
    author_email='tom_roberts.1992@hotmail.co.uk',
    license='GNU GPL V3.0',
    packages=['pics2word'],
    install_requires=['docx','python-docx','datetime'],
    python_requires='>3.5.2',
    include_package_data=True,
    zip_safe=True,
    long_description=readme(),
    entry_points = {
        'console_scripts': ['pics2word=pics2word.__init__:main'],
    },
    classifiers=[
    'Development Status :: 1 - Planning',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Programming Language :: Python :: 3.6',
    'Environment :: Console',
    'Topic :: Office/Business :: Office Suites',
    ]
)